package org.codehaus.mojo.buildhelper;

/*
 * The MIT License
 *
 * Copyright (c) 2004, The Codehaus
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugins.annotations.LifecyclePhase;
import org.apache.maven.plugins.annotations.Mojo;
import org.apache.maven.plugins.annotations.Parameter;

import java.net.InetAddress;
import java.net.UnknownHostException;

/**
 * Retrieve current hostname and place it under a configurable project property
 *
 * @author <a href="nhoj.patrick@gmail.com">John Patrick</a>
 * @since 3.0.0
 */
@Mojo( name = "hostname", defaultPhase = LifecyclePhase.PROCESS_TEST_CLASSES, threadSafe = true )
public class HostnameMojo
    extends AbstractDefinePropertyMojo
{

    /**
     * The name of the property in which to store the hostname.
     */
    @Parameter( defaultValue = "hostname" )
    private String hostnameProperty;

    public void execute()
        throws MojoExecutionException
    {
        try
        {
            defineProperty( this.hostnameProperty, InetAddress.getLocalHost().getHostName() );
        }
        catch ( UnknownHostException e )
        {
            throw new MojoExecutionException( "Unable to retrieve hostname.", e );
        }
    }
}
